==> nakenchat-2.22.tar.gz: nakenchat-2.22/src/passwd_sqlite.c <==
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sqlite3.h>

#include "globals.h"
#include "functions.h"

#ifdef SQLITE_NO_V2
#define sqlite3_open_v2 sqlite3_open
#define sqlite3_prepare_v2 sqlite3_prepare
#endif

int passwd_check(char *name, int ID)
{
const unsigned char *cvalue;
const char *cname;
sqlite3_stmt *rs=0;
const char *sqlp=0;
char *password;
char sql[256];
char esc_name[(MAX_USER_NAME_LENGTH*2)+(PASSWORD_LENGTH*2)+1];
sqlite3 *db;
int i,l,c;
int retcode=2;

  if (strlen(name)>MAX_USER_NAME_LENGTH+1+PASSWORD_LENGTH) return 2;
  sql_esc(name, esc_name);

  if (strlen(esc_name)>sizeof(sql)-sizeof("select * from users where lower(username)=lower('')")) return 1;

  l=0; i=-1;
  while (esc_name[l]!=0)
  {
    if (esc_name[l]=='=') i=l; 
    l++;
  }

  if (i==-1)
  { password=esc_name+l; }
    else
  {
    esc_name[i]=0;
    password=esc_name+i+1;

    l=0; i=-1;
    while (name[l]!=0)
    {
      if (name[l]=='=') i=l; 
      l++;
    }
    name[i]=0;
  }

#ifdef DEBUG
  if (debug==1) printf("Sqlite: Checking username '%s', password '%s'\n", name, password);
#endif

  /* if (sqlite3_open_v2(passwd,&db,SQLITE_OPEN_READONLY,NULL)!=0) */
  /* keeping the database open could be global maybe.. and save resources.. */
  if (sqlite3_open(passwd,&db)!=SQLITE_OK)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Couldn't open password file.\n");
#endif
    sqlite3_close(db);
    return 1;
  }

  sprintf(sql,"select * from users where lower(username)=lower('%s')",esc_name);

#ifdef DEBUG
  if (debug==1) printf("sqlite: %s\n",sql);
#endif

  // printf("%s   password='%s'\n",sql,password);

  if ((l=sqlite3_prepare_v2(db,sql,sizeof(sql),&rs,&sqlp))!=SQLITE_OK)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Illegal Query (%d) %s. %s\n",l,sql,sqlite3_errmsg(db));
#endif
    sqlite3_close(db);
    return 1;
  }

  l=sqlite3_step(rs);
  if (l==SQLITE_ERROR)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Step error %s.\n",sql);
#endif
    sqlite3_finalize(rs);
    sqlite3_close(db);
    return 1;
  }

  if (l==SQLITE_ROW)
  {
    l=sqlite3_column_count(rs);
    retcode=1;

    for (c=0; c<l; c++)
    {
      cname=sqlite3_column_name(rs,c);
      cvalue=sqlite3_column_text(rs,c);

      if (strcasecmp((char *)cname,"password")==0)
      {
        if (strcmp((char *)cvalue,password)!=0) break;

        retcode=0;
      }

      if (strcasecmp(cname,"level")==0)
      {
#ifdef DEBUG
        if (debug==1) printf("Setting level '%s' %d\n", cvalue, atoi((char *)cvalue));
#endif
        users[ID]->user_level=atoi((char *)cvalue);
      }
        else
      if (strcasecmp(cname,"emote")==0)
      {
        users[ID]->emote_char=cvalue[0];
      }
        else
      if (strcasecmp(cname,"flags")==0)
      {
        // t=atoi(cvalue);
        // l=0xffffffff^(1|2|16);
        users[ID]->uflags=users[ID]->uflags&(0xffffffff^(1|2|16));
        users[ID]->uflags=users[ID]->uflags|atoi((char *)cvalue);
      }
        else
      if (strcasecmp(cname,"default_channel")==0)
      {
        i=channel(ID,(char *)cvalue);
        if (i>=0)
        {
          users[ID]->channel=i;
          users[ID]->uflags=users[ID]->uflags&65531;
        }
      }
    }
  }

  sqlite3_finalize(rs);
  sqlite3_close(db);

  return retcode;
}

int passwd_reg_name(int ID, char *password)
{
sqlite3_stmt *rs;
const char *sqlp;
// char *password;
char sql[256];
sqlite3 *db;
int l;
char esc_emote[3];
char esc_name[(MAX_USER_NAME_LENGTH*2)+1];
char esc_password[(PASSWORD_LENGTH*2)+1];
char esc_channel[(CHANNEL_NAME_LEN*2)+1];
int flags;

/* insert into users values (
 * 'username',
 * 'password',
 * '%',
 * level,
 * 'channel',
 * flags);
 *
 * update users set
 * password='password',
 * emote='%',
 * level=level,
 * channel='channel',
 * flags=flags;
 */

#ifdef DEBUG
  if (debug==1) { printf("Register with password for user %d\n", ID); }
#endif

  if (strlen(password)>16) return -2;

  /* keeping the database open could be global maybe.. and save resources.. */
  if (sqlite3_open(passwd,&db)!=SQLITE_OK)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Couldn't open password file.\n");
#endif
    sqlite3_close(db);
    return 1;
  }

  flags=users[ID]->uflags;

  sql_esc(users[ID]->username, esc_name);
  sql_esc(password, esc_password);

  if (users[ID]->channel!=0)
  { sql_esc(channels[users[ID]->channel], esc_channel); }
    else
  { esc_channel[0]=0; }

  if (users[ID]->emote_char=='\'' || users[ID]->emote_char=='\\')
  { esc_emote[0]='\\'; esc_emote[1]=users[ID]->emote_char; esc_emote[2]=0; }
    else
  { esc_emote[0]=users[ID]->emote_char; esc_emote[1]=0; }

  sprintf(sql, "update users set "
               "password='%s',"
               "emote='%s',"
               "level=%d,"
               "channel='%s',"
               "flags=%d "
               "where username='%s'",
               esc_password, esc_emote, users[ID]->user_level,
               esc_channel, flags, esc_name);

#ifdef DEBUG
  if (debug==1) { printf("Sqlite: %s\n", sql); }
#endif

  if ((l=sqlite3_prepare_v2(db,sql,sizeof(sql),&rs,&sqlp))!=SQLITE_OK)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Illegal Query (%d) %s. %s\n",l,sql,sqlite3_errmsg(db));
#endif
    sqlite3_close(db);
    return 1;
  }

  l=sqlite3_step(rs);

  if (l==SQLITE_ERROR)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Step error %s. %s.\n",sql,sqlite3_errmsg(db));
#endif
    sqlite3_finalize(rs);
    sqlite3_close(db);
    return 1;
  }

#ifdef DEBUG
  if (debug==1) { printf("Sqlite: update %d\n", l); }
#endif

  if (l==SQLITE_DONE && sqlite3_changes(db)>0)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: update SQLITE_DONE\n");
#endif
    sqlite3_finalize(rs);
    sqlite3_close(db);
  }
    else
  {
    sqlite3_finalize(rs);

    sprintf(sql, "insert into users values ("
                 "'%s',"
                 "'%s',"
                 "'%s',"
                 "%d,"
                 "'%s',"
                 "%d)",
              esc_name,
              esc_password,
              esc_emote, users[ID]->user_level, esc_channel, flags);

#ifdef DEBUG
  if (debug==1) { printf("Sqlite: insert %s\n", sql); }
#endif

    if ((l=sqlite3_prepare_v2(db,sql,sizeof(sql),&rs,&sqlp))!=SQLITE_OK)
    {
#ifdef DEBUG
      if (debug==1) printf("Sqlite: Illegal Query (%d) %s. %s\n",l,sql,sqlite3_errmsg(db));
#endif
      sqlite3_close(db);
      return 1;
    }

    l=sqlite3_step(rs);
    if (l==SQLITE_ERROR)
    {
#ifdef DEBUG
      if (debug==1) printf("Sqlite: Step error %s. %s.\n",sql,sqlite3_errmsg(db));
#endif
      sqlite3_finalize(rs);
      sqlite3_close(db);
      return 1;
    }

#ifdef DEBUG
  if (debug==1) { printf("Sqlite: It works? l=%d\n",l); }
#endif

    sqlite3_finalize(rs);
    sqlite3_close(db);
  }

  return 0;
}

==> nakenchat-3.00b1.tar.gz: nakenchat-3.00b1/src/passwd_sqlite.c <==
/**
 *  Naken Chat
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPL
 *
 * Copyright 1998-2013 by Michael Kohn
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sqlite3.h>

#include "general.h"
#include "nakenchat.h"
#include "passwd.h"

#ifdef SQLITE_NO_V2
#define sqlite3_open_v2 sqlite3_open
#define sqlite3_prepare_v2 sqlite3_prepare
#endif

int passwd_check(char *name, int ID)
{
const unsigned char *cvalue;
const char *cname;
sqlite3_stmt *rs=0;
const char *sqlp=0;
char *password;
char sql[256];
char esc_name[(MAX_USER_NAME_LENGTH*2)+(PASSWORD_LENGTH*2)+1];
sqlite3 *db;
int i,l,c;
int retcode=2;

  if (strlen(name)>MAX_USER_NAME_LENGTH+1+PASSWORD_LENGTH) return 2;
  sql_esc(name, esc_name);

  if (strlen(esc_name)>sizeof(sql)-sizeof("select * from users where lower(username)=lower('')")) return 1;

  l=0; i=-1;
  while (esc_name[l]!=0)
  {
    if (esc_name[l]=='=') i=l; 
    l++;
  }

  if (i==-1)
  { password=esc_name+l; }
    else
  {
    esc_name[i]=0;
    password=esc_name+i+1;

    l=0; i=-1;
    while (name[l]!=0)
    {
      if (name[l]=='=') i=l; 
      l++;
    }
    name[i]=0;
  }

#ifdef DEBUG
  if (debug==1) printf("Sqlite: Checking username '%s', password '%s'\n", name, password);
#endif

  /* if (sqlite3_open_v2(passwd,&db,SQLITE_OPEN_READONLY,NULL)!=0) */
  /* keeping the database open could be global maybe.. and save resources.. */
  if (sqlite3_open(server_info->passwd,&db)!=SQLITE_OK)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Couldn't open password file.\n");
#endif
    sqlite3_close(db);
    return 1;
  }

  sprintf(sql,"select * from users where lower(username)=lower('%s')",esc_name);

#ifdef DEBUG
  if (debug==1) printf("sqlite: %s\n",sql);
#endif

  // printf("%s   password='%s'\n",sql,password);

  if ((l=sqlite3_prepare_v2(db,sql,sizeof(sql),&rs,&sqlp))!=SQLITE_OK)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Illegal Query (%d) %s. %s\n",l,sql,sqlite3_errmsg(db));
#endif
    sqlite3_close(db);
    return 1;
  }

  l=sqlite3_step(rs);
  if (l==SQLITE_ERROR)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Step error %s.\n",sql);
#endif
    sqlite3_finalize(rs);
    sqlite3_close(db);
    return 1;
  }

  if (l==SQLITE_ROW)
  {
    l=sqlite3_column_count(rs);
    retcode=1;

    for (c=0; c<l; c++)
    {
      cname=sqlite3_column_name(rs,c);
      cvalue=sqlite3_column_text(rs,c);

      if (strcasecmp((char *)cname,"password")==0)
      {
        if (strcmp((char *)cvalue,password)!=0) break;

        retcode=0;
      }

      if (strcasecmp(cname,"level")==0)
      {
#ifdef DEBUG
        if (debug==1) printf("Setting level '%s' %d\n", cvalue, atoi((char *)cvalue));
#endif
        users[ID]->user_level=atoi((char *)cvalue);
      }
        else
      if (strcasecmp(cname,"emote")==0)
      {
        users[ID]->emote_char=cvalue[0];
      }
        else
      if (strcasecmp(cname,"flags")==0)
      {
        // t=atoi(cvalue);
        // l=0xffffffff^(1|2|16);
        users[ID]->uflags=users[ID]->uflags&(0xffffffff^(1|2|16));
        users[ID]->uflags=users[ID]->uflags|atoi((char *)cvalue);
      }
        else
      if (strcasecmp(cname,"default_channel")==0)
      {
        i=channel(ID,(char *)cvalue);
        if (i>=0)
        {
          users[ID]->channel=i;
          users[ID]->uflags=users[ID]->uflags&65531;
        }
      }
    }
  }

  sqlite3_finalize(rs);
  sqlite3_close(db);

  return retcode;
}

int passwd_reg_name(int ID, char *password)
{
sqlite3_stmt *rs;
const char *sqlp;
// char *password;
char sql[256];
sqlite3 *db;
int l;
char esc_emote[3];
char esc_name[(MAX_USER_NAME_LENGTH*2)+1];
char esc_password[(PASSWORD_LENGTH*2)+1];
char esc_channel[(CHANNEL_NAME_LEN*2)+1];
int flags;

/* insert into users values (
 * 'username',
 * 'password',
 * '%',
 * level,
 * 'channel',
 * flags);
 *
 * update users set
 * password='password',
 * emote='%',
 * level=level,
 * channel='channel',
 * flags=flags;
 */

#ifdef DEBUG
  if (debug==1) { printf("Register with password for user %d\n", ID); }
#endif

  if (strlen(password)>16) return -2;

  /* keeping the database open could be global maybe.. and save resources.. */
  if (sqlite3_open(server_info->passwd,&db)!=SQLITE_OK)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Couldn't open password file.\n");
#endif
    sqlite3_close(db);
    return 1;
  }

  flags=users[ID]->uflags;

  sql_esc(users[ID]->username, esc_name);
  sql_esc(password, esc_password);

  if (users[ID]->channel!=0)
  { sql_esc(server_info->channels[users[ID]->channel], esc_channel); }
    else
  { esc_channel[0]=0; }

  if (users[ID]->emote_char=='\'' || users[ID]->emote_char=='\\')
  { esc_emote[0]='\\'; esc_emote[1]=users[ID]->emote_char; esc_emote[2]=0; }
    else
  { esc_emote[0]=users[ID]->emote_char; esc_emote[1]=0; }

  sprintf(sql, "update users set "
               "password='%s',"
               "emote='%s',"
               "level=%d,"
               "channel='%s',"
               "flags=%d "
               "where username='%s'",
               esc_password, esc_emote, users[ID]->user_level,
               esc_channel, flags, esc_name);

#ifdef DEBUG
  if (debug==1) { printf("Sqlite: %s\n", sql); }
#endif

  if ((l=sqlite3_prepare_v2(db,sql,sizeof(sql),&rs,&sqlp))!=SQLITE_OK)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Illegal Query (%d) %s. %s\n",l,sql,sqlite3_errmsg(db));
#endif
    sqlite3_close(db);
    return 1;
  }

  l=sqlite3_step(rs);

  if (l==SQLITE_ERROR)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: Step error %s. %s.\n",sql,sqlite3_errmsg(db));
#endif
    sqlite3_finalize(rs);
    sqlite3_close(db);
    return 1;
  }

#ifdef DEBUG
  if (debug==1) { printf("Sqlite: update %d\n", l); }
#endif

  if (l==SQLITE_DONE && sqlite3_changes(db)>0)
  {
#ifdef DEBUG
    if (debug==1) printf("Sqlite: update SQLITE_DONE\n");
#endif
    sqlite3_finalize(rs);
    sqlite3_close(db);
  }
    else
  {
    sqlite3_finalize(rs);

    sprintf(sql, "insert into users values ("
                 "'%s',"
                 "'%s',"
                 "'%s',"
                 "%d,"
                 "'%s',"
                 "%d)",
              esc_name,
              esc_password,
              esc_emote, users[ID]->user_level, esc_channel, flags);

#ifdef DEBUG
  if (debug==1) { printf("Sqlite: insert %s\n", sql); }
#endif

    if ((l=sqlite3_prepare_v2(db,sql,sizeof(sql),&rs,&sqlp))!=SQLITE_OK)
    {
#ifdef DEBUG
      if (debug==1) printf("Sqlite: Illegal Query (%d) %s. %s\n",l,sql,sqlite3_errmsg(db));
#endif
      sqlite3_close(db);
      return 1;
    }

    l=sqlite3_step(rs);
    if (l==SQLITE_ERROR)
    {
#ifdef DEBUG
      if (debug==1) printf("Sqlite: Step error %s. %s.\n",sql,sqlite3_errmsg(db));
#endif
      sqlite3_finalize(rs);
      sqlite3_close(db);
      return 1;
    }

#ifdef DEBUG
  if (debug==1) { printf("Sqlite: It works? l=%d\n",l); }
#endif

    sqlite3_finalize(rs);
    sqlite3_close(db);
  }

  return 0;
}
